@@ -1,16 +1,36 @@
-geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "Age"), boxes="Age", tick.scale="myr", user.scale, cex.age=0.3, cex.ts=0.3, cex.tip=0.3, width=1, ts.col=TRUE, ranges=FALSE, vers="ICS2013", x.lim, quat.rm=FALSE,...){
+geoscalePhylo<-function(tree, ages, direction="rightwards",units=c("Period", "Epoch", "Age"), boxes="Age", tick.scale="myr", user.scale, cex.age=0.3, cex.ts=0.3, cex.tip=0.3, width=1, label.offset,ts.col=TRUE, vers="ICS2013", x.lim, quat.rm=FALSE,erotate,arotate,urotate,...){
   
-  if(boxes == "User" && all(units != "User")){
+  options <- as.list(match.call())
+   if(any(names(options) == "type")){
+     if(all(options$type != c("phylogram","cladogram","p","c"))){
+       return(cat("type must be either 'phylogram' or 'cladogram'."))
+     }
+   }
+  
+   if(all(direction != c("rightwards","upwards"))){
+      return(cat("direction must be either 'rightwards' or 'upwards', here set to 'rightwards'."))
+    }
+  
+  if(is.null(tree$root.time)){     
+    return(cat("\n tree$root.time is missing, check tree is time scaled."))
+  } else {root.age <- tree$root.time}
+  
+  if(boxes == "User" && any(units != "User")){
     boxes <- "no"
   }
-
+  
+  if(all(boxes != units)){
+    boxes <- "no"
+  }
+  
   if(tick.scale == "User" && all(units != "User")){
     tick.scale <- "myr"
   }
    
-  if(missing(ages) == TRUE && ranges == TRUE){
-    cat("\n ages file is not provided, ranges is to FALSE")
-      ranges <- FALSE
+  if(missing(ages) == FALSE){
+      ranges <- TRUE
+  } else{
+    ranges <- FALSE
   }
   
   if(missing(user.scale) & any(units == "User")){
@@ -28,14 +48,12 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
         user.scale <- cbind(user.scale,Midpoint)  
   }
   
-  if(length(units) == 1){
-  	ts.width=0.15
-  } else if(length(units) == 2){
-  	ts.width=0.2
-  } else if(length(units) >= 3){
-  	ts.width=0.25
+  if(all(units != "Age") && boxes == "Age"){
+    boxes <- "no"
   }
   
+  units <- paste(toupper(substring(units,1,1)),substring(units,2),sep="")
+    
   # Standardizing the names of temporal units
   
   units[units == "Eonothem"] <- "Eon"
@@ -51,13 +69,20 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
   boxes[boxes == "System"] <- "Period"  
   boxes[boxes == "Stage"] <- "Age"
       
+  if(length(units) == 1){
+    ts.width=0.15
+  } else if(length(units) == 2){
+    ts.width=0.2
+  } else if(length(units) >= 3){
+    ts.width=0.25
+  }
+
   if(ranges == TRUE && missing(ages) == FALSE){
      missing.tip.names <- setdiff(tree$tip.label,row.names(ages))
       if(length(missing.tip.names) > 0){            
         cat(paste("\n",missing.tip.names,"not present in ages file, ranges set to FALSE"))
         cat("\n ranges set to FALSE")
           ranges <- FALSE
-        
       }    
   }
    
@@ -71,24 +96,40 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
   tscale.data<-matrix(ncol=3,nrow=6)
     colnames(tscale.data) <-c("srt","Depth","size")
     rownames(tscale.data) <-c("Eon","Era","Period","Epoch","Age","User")
-      if(upwards == TRUE){
+      if(direction == "upwards"){
         tscale.data[,"srt"] <- c(90,90,90,0,0,0)
       } else tscale.data[,"srt"] <- c(0,0,0,90,90,90)
       
       tscale.data[,"Depth"] <- c(1,1,1,2,3.5,3.5)
       tscale.data[,"size"] <- c(1,1,1,0.8,0.8,0.8)
 
+  ## ROTATING THE NAMES
+  
+  if(!missing(erotate) && !is.numeric(erotate)){
+    return(cat("\n value for protate must be numeric."))
+  }
+  if(!missing(arotate) && !is.numeric(arotate)){
+    return(cat("\n value for arotate must be numeric."))
+  }
+  if(!missing(urotate) && !is.numeric(urotate)){
+    return(cat("\n value for urotate must be numeric."))
+  }
+  
+  if(!missing(erotate)){
+    tscale.data["Epoch","srt"] <- erotate
+  }
+  if(!missing(arotate)){
+    tscale.data["Age","srt"] <- arotate
+  }
+  if(!missing(urotate)){
+    tscale.data["User","srt"] <- urotate
+  }
+  
   ## GEOLOGICAL RANGES OF TAXA
 
   units<-rownames(tscale.data)[sort(match(units,rownames(tscale.data)),decreasing=T)] 
-  
-   if(is.null(tree$root.time)){     
-      return(cat("\n tree$root.time is missing, check tree is time scaled."))
-   } else {root.age <- tree$root.time}
-  
-  # if x.lim set by user -  
-  
-  if(!missing(x.lim)){
+    
+  if(!missing(x.lim) ){
     x.lim <- sort(root.age - x.lim)
   } else if(ranges == TRUE && !missing(ages) && missing(x.lim)){
     x.lim <- (root.age - min(ages)) + diff(range(ages))*0.05
@@ -99,14 +140,24 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
     timescale<-timescale[order(timescale[,1],decreasing=T),]
     	timescale.rescaled <- timescale
   		timescale.rescaled[,c("Start","End","Midpoint")] <- root.age - timescale[,c("Start","End","Midpoint")]
-	
-    offset<-array(dim=length(tree$tip.label),data=1)
 
+  first_la <- tree$root.time - dist.nodes(tree)[1,Ntip(tree)+1]
+	
     if(ranges==TRUE && missing(ages) == FALSE){
-      
-      offset.correction <- diff(range(ages)) * 0.01 
-      taxon.ranges <- root.age - ages[,c("FAD","LAD")]
-       offset <- array(dim=length(ages[,"FAD"]),data=(ages[,"FAD"] - ages[,"LAD"])+offset.correction)
+      offset<-array(dim=length(tree$tip.label),data=1)
+       offset.correction <- diff(range(ages)) * 0.01 
+        taxon.ranges <- root.age - ages[,c("FAD","LAD")]
+         if(first_la != ages[1,"LAD"]){
+          if(!missing(label.offset)){
+           offset <- array(dim=length(ages[,"FAD"]),data=(ages[,"FAD"] - ages[,"LAD"])+label.offset)
+          } else {
+           offset <- array(dim=length(ages[,"FAD"]),data=(ages[,"FAD"] - ages[,"LAD"])+offset.correction)
+          }
+         }
+    } else if(!missing(label.offset)){
+      offset <- label.offset 
+    } else {
+      offset = 1
     }
 
   ### ADDING A TIMESCALE
@@ -131,8 +182,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
         age.name<-sort(unique(c(age.name[,"Start"],age.name[,"End"])))
         	age.position<- tick.position <- root.age - age.name
             lwd=1
-              col="black"
-      
+              col="black" 
     } 
     
     if(tick.scale == "User"){     
@@ -144,13 +194,13 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
    }
   
   # Plotting the tree vertically
-  if(upwards == TRUE){
+  if(direction == "upwards"){
     
     # ADDING THE TIME SCALE
     par(fig=c(0,ts.width,0,1))
      par(mar=c(3,1,2,5))
     
-        plot.phylo(tree,plot=FALSE,no.margin=T,y.lim=x.lim,direction="upwards",...)
+        plot.phylo(tree,plot=FALSE,no.margin=T,y.lim=x.lim,direction="upwards",cex=cex.tip,...)
           
           timescale.rescaled.names <- timescale.rescaled
            timescale.rescaled.names <- timescale.rescaled.names[timescale.rescaled.names[,"End"] > par()$usr[3],]
@@ -203,7 +253,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
     par(fig=c(ts.width,1,0,1),new=T)
      par(mar=c(3,0,2,2))
     
-      plot.phylo(tree,plot=FALSE,no.margin=T,y.lim=x.lim,direction="upwards",...)
+      plot.phylo(tree,plot=FALSE,no.margin=T,y.lim=x.lim,direction="upwards",cex=cex.tip,...)
         lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
     
       if (!missing(boxes) && boxes != "no" && boxes !="n"){
@@ -220,7 +270,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
     
         plot.phylo(tree,label.offset=offset,edge.width=width,no.margin=T,y.lim=x.lim,cex=cex.tip,direction="upwards",...)
           if (ranges == TRUE){
-            segments(lastPP$xx[c(1:length(tree$tip.label))],taxon.ranges[,"FAD"],lastPP$xx[c(1:length(tree$tip.label))],taxon.ranges[,"LAD"],col="black",lwd=width*2)
+            par(lend=1); segments(lastPP$xx[c(1:length(tree$tip.label))],taxon.ranges[,"FAD"],lastPP$xx[c(1:length(tree$tip.label))],taxon.ranges[,"LAD"],col="black",lwd=width*2)
           }
     
     if(units[1] == "User"){
@@ -236,7 +286,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
     par(fig=c(0,1,0,ts.width))
     par(mar=c(1,3,0,2))
 
-	    plot.phylo(tree,plot=FALSE,no.margin=T,x.lim=x.lim,direction="rightwards",...)
+	    plot.phylo(tree,plot=FALSE,no.margin=T,x.lim=x.lim,direction="rightwards",cex=cex.tip,...)
     
         timescale.rescaled.names <- timescale.rescaled
          timescale.rescaled.names <- timescale.rescaled.names[timescale.rescaled.names[,"End"] > par()$usr[1],]
@@ -288,7 +338,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
     par(fig=c(0,1,ts.width,1),new=T)
     par(mar=c(0,3,2,2))
   
-      plot.phylo(tree,plot=FALSE,no.margin=T,x.lim=x.lim,...)
+      plot.phylo(tree,plot=FALSE,no.margin=T,x.lim=x.lim,cex=cex.tip,...)
         lastPP <- get("last_plot.phylo", envir = .PlotPhyloEnv)
   
       if (!missing(boxes) && boxes != "no" && boxes !="n"){
@@ -302,7 +352,7 @@ geoscalePhylo<-function(tree, ages, upwards=FALSE, units=c("Period", "Epoch", "A
   
       plot.phylo(tree,label.offset=offset,edge.width=width,no.margin=T,x.lim=x.lim,cex=cex.tip,...)
         if (ranges == TRUE){
-          segments(taxon.ranges[,"FAD"],lastPP$yy[c(1:length(tree$tip.label))],taxon.ranges[,"LAD"],lastPP$yy[c(1:length(tree$tip.label))],col="black",lwd=width*2)
+          par(lend=1); segments(taxon.ranges[,"FAD"],lastPP$yy[c(1:length(tree$tip.label))],taxon.ranges[,"LAD"],lastPP$yy[c(1:length(tree$tip.label))],col="black",lwd=width*2)
         }
   
   if(units[1] == "User"){
